@@ -91,15 +91,15 @@
 GST_DEBUG_CATEGORY_STATIC (clutter_gst_video_sink_debug);
 #define GST_CAT_DEFAULT clutter_gst_video_sink_debug
 
-#define CLUTTER_GST_DEFAULT_PRIORITY G_PRIORITY_HIGH_IDLE
+#define CLUTTER_GST_DEFAULT_PRIORITY CLUTTER_PRIORITY_REDRAW
 
 #define BASE_SINK_CAPS "{ AYUV,"                \
   "YV12,"                                       \
   "I420,"                                       \
   "RGBA,"                                       \
   "BGRA,"                                       \
-  "RGBX,"                                       \
-  "BGRX,"                                       \
+  "RGBx,"                                       \
+  "BGRx,"                                       \
   "RGB,"                                        \
   "BGR,"                                        \
   "NV12 }"
@@ -400,12 +400,12 @@ add_layer_cache_entry (ClutterGstVideoSink *sink,
 
   default_source =
     g_strdup_printf ("  cogl_layer *= clutter_gst_sample_video%i "
-                     "(cogl_tex_coord%i_in.st);\n",
+                     "(cogl_tex_coord%i_in.st * clutter_gst_transform);\n",
                      priv->video_start,
                      priv->video_start);
   entry->default_sample_snippet =
     cogl_snippet_new (COGL_SNIPPET_HOOK_LAYER_FRAGMENT,
-                      NULL, /* declarations */
+                      "uniform mat2 clutter_gst_transform;\n",
                       default_source);
   g_free (default_source);
 
@@ -745,10 +745,38 @@ clutter_gst_video_sink_attach_frame (ClutterGstVideoSink *sink,
 /* Color balance */
 
 static const gchar *no_color_balance_shader =
+  "\n"
   "#define clutter_gst_get_corrected_color_from_yuv(arg) (arg)\n"
   "#define clutter_gst_get_corrected_color_from_rgb(arg) (arg)\n";
 
 static const gchar *color_balance_shader =
+  "\n"
+  "vec3\n"
+  "clutter_gst_rgb_to_yuv (vec3 rgb)\n"
+  "{\n"
+  "  return mat3 (0.2126, -0.114626,  0.5,\n"
+  "               0.7152, -0.385428, -0.454153,\n"
+  "               0.0722,  0.5,       0.045847 ) * rgb;\n"
+  "}\n"
+  "\n"
+  /* " /\* bt601 version *\/\n" */
+  /* "vec3\n" */
+  /* "clutter_gst_rgb_to_yuv (vec3 rgb)\n" */
+  /* "{\n" */
+  /* "  return mat3 (0.299,  0.5,      -0.168736,\n" */
+  /* "               0.587, -0.418688, -0.331264,\n" */
+  /* "               0.114, -0.081312,  0.5      ) * rgb;\n" */
+  /* "}\n" */
+  /* "\n" */
+  /* " /\* bt2020 version *\/\n" */
+  /* "vec3\n" */
+  /* "clutter_gst_rgb_to_bt2020 (vec3 rgb)\n" */
+  /* "{\n" */
+  /* "  return mat3 (0.2627, -0.139630,  0.503380,\n" */
+  /* "               0.6780, -0.360370, -0.462893,\n" */
+  /* "               0.0593,  0.5,      -0.040486 ) * rgb;\n" */
+  /* "}\n" */
+
   "vec3\n"
   "clutter_gst_get_corrected_color_from_yuv (vec3 yuv)\n"
   "{\n"
@@ -761,11 +789,11 @@ static const gchar *color_balance_shader =
   "vec3\n"
   "clutter_gst_get_corrected_color_from_rgb (vec3 rgb)\n"
   "{\n"
-  "  vec3 yuv = clutter_gst_yuv_srgb_to_bt601 (rgb);\n"
+  "  vec3 yuv = clutter_gst_rgb_to_yuv (rgb);\n"
   "  vec3 corrected_yuv = vec3 (texture2D (cogl_sampler%i, vec2 (yuv[0], 0)).a,\n"
   "                             texture2D (cogl_sampler%i, vec2 (yuv[2], yuv[1])).a,\n"
   "                             texture2D (cogl_sampler%i, vec2 (yuv[2], yuv[1])).a);\n"
-  "  return clutter_gst_yuv_bt601_to_srgb (corrected_yuv);\n"
+  "  return clutter_gst_yuv_to_rgb (corrected_yuv);\n"
   "}\n";
 
 static void
@@ -864,86 +892,64 @@ clutter_gst_video_sink_setup_balance (ClutterGstVideoSink *sink,
 
 /* YUV <-> RGB conversions */
 
-static const gchar *color_conversions_shaders =
+static const gchar *no_color_conversions_shader =
   "\n"
-  "/* These conversion functions take : */\n"
+  "#define clutter_gst_yuv_to_rgb(color) (color)\n";
+static const gchar *color_conversions_shader =
+  "\n"
+  "/* This conversion functions take : */\n"
   "/*   Y = [0, 1] */\n"
   "/*   U = [-0.5, 0.5] */\n"
   "/*   V = [-0.5, 0.5] */\n"
-  "vec3\n"
-  "clutter_gst_yuv_bt601_to_srgb (vec3 yuv)\n"
-  "{\n"
-  "  return mat3 (1.0,    1.0,      1.0,\n"
-  "               0.0,   -0.344136, 1.772,\n"
-  "               1.402, -0.714136, 0.0   ) * yuv;\n"
-  "}\n"
-  "\n"
-  "vec3\n"
-  "clutter_gst_yuv_bt709_to_srgb (vec3 yuv)\n"
-  "{\n"
-  "  return mat3 (1.0,     1.0,      1.0,\n"
-  "               0.0,    -0.187324, 1.8556,\n"
-  "               1.5748, -0.468124, 0.0    ) * yuv;\n"
-  "}\n"
-  "\n"
-  "vec3\n"
-  "clutter_gst_yuv_bt2020_to_srgb (vec3 yuv)\n"
-  "{\n"
-  "  return mat3 (1.0,     1.0,      1.0,\n"
-  "               0.0,     0.571353, 1.8814,\n"
-  "               1.4746,  0.164553, 0.0    ) * yuv;\n"
-  "}\n"
-  "/* Original transformation, still no idea where these values come from... */\n"
-  "vec3\n"
-  "clutter_gst_yuv_originalyuv_to_srgb (vec3 yuv)\n"
-  "{\n"
-  "  return mat3 (1.0,         1.0,      1.0,\n"
-  "               0.0,        -0.390625, 2.015625,\n"
-  "               1.59765625, -0.8125,   0.0      ) * yuv;\n"
-  "}\n"
-  "\n"
-  "vec3\n"
-  "clutter_gst_yuv_srgb_to_bt601 (vec3 rgb)\n"
-  "{\n"
-  "  return mat3 (0.299,  0.5,      -0.168736,\n"
-  "               0.587, -0.418688, -0.331264,\n"
-  "               0.114, -0.081312,  0.5      ) * rgb;\n"
-  "}\n"
-  "\n"
-  "vec3\n"
-  "clutter_gst_yuv_srgb_to_bt709 (vec3 rgb)\n"
-  "{\n"
-  "  return mat3 (0.2126, -0.114626,  0.5,\n"
-  "               0.7152, -0.385428, -0.454153,\n"
-  "               0.0722,  0.5,       0.045847 ) * rgb;\n"
-  "}\n"
-  "\n"
-  "vec3\n"
-  "clutter_gst_yuv_srgb_to_bt2020 (vec3 rgb)\n"
-  "{\n"
-  "  return mat3 (0.2627, -0.139630,  0.503380,\n"
-  "               0.6780, -0.360370, -0.462893,\n"
-  "               0.0593,  0.5,      -0.040486 ) * rgb;\n"
-  "}\n"
-  "\n"
-  "#define clutter_gst_default_yuv_to_srgb(arg) clutter_gst_yuv_%s_to_srgb(arg)\n"
+  "uniform mat3 clutter_gst_yuv_to_rgb_;\n"
+  "#define clutter_gst_yuv_to_rgb(yuv) (clutter_gst_yuv_to_rgb_ * (yuv))\n"
   "\n";
 
-static const char *
-_gst_video_color_matrix_to_string (GstVideoColorMatrix matrix)
+static const float *
+_matrix_from_colorimetry (GstVideoColorMatrix matrix)
 {
+  static const float bt601[] = { 1.0,    1.0,      1.0,
+                                 0.0,   -0.344136, 1.772,
+                                 1.402, -0.714136, 0.0    };
+  static const float bt709[] = { 1.0,     1.0,      1.0,
+                                 0.0,    -0.187324, 1.8556,
+                                 1.5748, -0.468124, 0.0     };
+#if GST_CHECK_VERSION(1, 6, 0)
+  static const float bt2020[] = { 1.0,     1.0,      1.0,
+                                  0.0,     0.571353, 1.8814,
+                                  1.4746,  0.164553, 0.0     };
+#endif
+#if 0
+  /* Original transformation from the clutter-gst 2.0/1.6 days.
+   * Still no idea where these values come from...
+   */
+  static const float original[] = { 1.0,         1.0,      1.0,
+                                    0.0,        -0.390625, 2.015625,
+                                    1.59765625, -0.8125,   0.0       };
+#endif
+
   switch (matrix)
     {
+#if GST_CHECK_VERSION(1, 6, 0)
+    case GST_VIDEO_COLOR_MATRIX_BT2020:
+      return bt2020;
+#endif
     case GST_VIDEO_COLOR_MATRIX_BT601:
-      return "bt601";
+      return bt601;
     case GST_VIDEO_COLOR_MATRIX_BT709:
-      return "bt709";
-
     default:
-      return "bt709";
+      return bt709;
     }
 }
 
+static const gchar *
+_shader_from_colorimetry (GstVideoColorMatrix matrix)
+{
+  if (matrix == GST_VIDEO_COLOR_MATRIX_RGB)
+    return no_color_conversions_shader;
+  return color_conversions_shader;
+}
+
 static void
 clutter_gst_video_sink_setup_conversions (ClutterGstVideoSink *sink,
                                           CoglPipeline *pipeline)
@@ -955,15 +961,45 @@ clutter_gst_video_sink_setup_conversions (ClutterGstVideoSink *sink,
 
   if (entry == NULL)
     {
-      char *source = g_strdup_printf (color_conversions_shaders,
-                                      _gst_video_color_matrix_to_string (matrix));
-
+      const char *source = _shader_from_colorimetry (matrix);
       entry = add_global_cache_entry (&snippet_cache, source, matrix);
-      g_free (source);
     }
 
   cogl_pipeline_add_snippet (pipeline, entry->vertex_snippet);
   cogl_pipeline_add_snippet (pipeline, entry->fragment_snippet);
+
+  if (matrix != GST_VIDEO_COLOR_MATRIX_RGB)
+    {
+      gint yuv_to_rgb_location =
+        cogl_pipeline_get_uniform_location (pipeline, "clutter_gst_yuv_to_rgb_");
+      cogl_pipeline_set_uniform_matrix (pipeline, yuv_to_rgb_location,
+                                        3, 1, FALSE,
+                                        _matrix_from_colorimetry (matrix));
+    }
+}
+
+/* Transformation for cropped videos */
+
+static void
+clutter_gst_video_sink_setup_transformation (ClutterGstVideoSink *sink,
+                                             CoglPipeline *pipeline)
+{
+  ClutterGstVideoSinkPrivate *priv = sink->priv;
+  float transform[] = { 1.0, 0.0,
+                        0.0, 1.0 };
+  gint transform_location;
+
+  if (priv->frame[0] != NULL)
+    {
+      transform[0] = (float) GST_VIDEO_INFO_COMP_WIDTH (&priv->info, 0) /
+        (float) cogl_texture_get_width (priv->frame[0]);
+      transform[3] = (float) GST_VIDEO_INFO_COMP_HEIGHT (&priv->info, 0) /
+        (float) cogl_texture_get_height (priv->frame[0]);
+    }
+  transform_location =
+    cogl_pipeline_get_uniform_location (pipeline, "clutter_gst_transform");
+  cogl_pipeline_set_uniform_matrix (pipeline, transform_location,
+                                    2, 1, FALSE, transform);
 }
 
 /**/
@@ -1283,13 +1319,16 @@ clutter_gst_rgb32_upload_gl (ClutterGstVideoSink *sink,
 
   upload_meta = gst_buffer_get_video_gl_texture_upload_meta (buffer);
   if (!upload_meta) {
-    GST_WARNING ("Buffer does not support GLTextureUploadMeta API");
+    GST_WARNING_OBJECT (sink,
+                        "Buffer does not support GLTextureUploadMeta API");
     return FALSE;
   }
 
   if (upload_meta->n_textures != priv->renderer->n_layers ||
       upload_meta->texture_type[0] != GST_VIDEO_GL_TEXTURE_TYPE_RGBA) {
-    GST_WARNING ("clutter-gst-video-sink only supports gl upload in a single RGBA texture");
+    GST_WARNING_OBJECT (sink,
+                        "clutter-gst-video-sink only supports gl upload in a"
+                        " single RGBA texture");
     return FALSE;
   }
 
@@ -1302,7 +1341,7 @@ clutter_gst_rgb32_upload_gl (ClutterGstVideoSink *sink,
         cogl_texture_set_components (priv->frame[i], COGL_TEXTURE_COMPONENTS_RGBA);
 
         if (!cogl_texture_allocate (priv->frame[i], NULL)) {
-          GST_WARNING ("Couldn't allocate cogl texture");
+          GST_WARNING_OBJECT (sink, "Couldn't allocate cogl texture");
           return FALSE;
         }
       }
@@ -1324,12 +1363,12 @@ clutter_gst_rgb32_upload_gl (ClutterGstVideoSink *sink,
 
 
   if (!cogl_texture_get_gl_texture (priv->frame[0], &gl_handle[0], NULL)) {
-    GST_WARNING ("Couldn't get gl texture");
+    GST_WARNING_OBJECT (sink, "Couldn't get gl texture");
     return FALSE;
   }
 
   if (!gst_video_gl_texture_upload_meta_upload (upload_meta, gl_handle)) {
-    GST_WARNING ("GL texture upload failed");
+    GST_WARNING_OBJECT (sink, "GL texture upload failed");
     return FALSE;
   }
 
@@ -1489,7 +1528,7 @@ clutter_gst_yv12_glsl_setup_pipeline (ClutterGstVideoSink *sink,
                          "  float v = texture2D (cogl_sampler%i, UV).a - 0.5;\n"
                          "  vec3 corrected = clutter_gst_get_corrected_color_from_yuv (vec3 (y, u, v));\n"
                          "  vec4 color;\n"
-                         "  color.rgb = clutter_gst_default_yuv_to_srgb (corrected);\n"
+                         "  color.rgb = clutter_gst_yuv_to_rgb (corrected);\n"
                          "  color.a = 1.0;\n"
                          "  return color;\n"
                          "}\n",
@@ -1557,7 +1596,7 @@ clutter_gst_ayuv_glsl_setup_pipeline (ClutterGstVideoSink *sink,
                            "  float v = color.a - 0.5;\n"
                            "  vec3 corrected = clutter_gst_get_corrected_color_from_yuv (vec3 (y, u, v));\n"
                            "  color.a = color.r;\n"
-                           "  color.rgb = clutter_gst_default_yuv_to_srgb (corrected);\n"
+                           "  color.rgb = clutter_gst_yuv_to_rgb (corrected);\n"
                            /* Premultiply the color */
                            "  color.rgb *= color.a;\n"
                            "  return color;\n"
@@ -1644,7 +1683,7 @@ clutter_gst_nv12_glsl_setup_pipeline (ClutterGstVideoSink *sink,
                          "  float u = uv.x;\n"
                          "  float v = uv.y;\n"
                          "  vec3 corrected = clutter_gst_get_corrected_color_from_yuv (vec3 (y, u, v));\n"
-                         "  color.rgb = clutter_gst_default_yuv_to_srgb (corrected);\n"
+                         "  color.rgb = clutter_gst_yuv_to_rgb (corrected);\n"
                          "  color.a = 1.0;\n"
                          "  return color;\n"
                          "}\n",
@@ -1673,19 +1712,19 @@ clutter_gst_nv12_upload (ClutterGstVideoSink *sink,
 
   priv->frame[0] =
     video_texture_new_from_data (priv->ctx,
-                                 GST_VIDEO_INFO_COMP_WIDTH (&priv->info, 0),
-                                 GST_VIDEO_INFO_COMP_HEIGHT (&priv->info, 0),
+                                 GST_VIDEO_FRAME_COMP_WIDTH (&frame, 0),
+                                 GST_VIDEO_FRAME_COMP_HEIGHT (&frame, 0),
                                  COGL_PIXEL_FORMAT_A_8,
-                                 priv->info.stride[0],
-                                 frame.data[0]);
+                                 GST_VIDEO_FRAME_COMP_STRIDE (&frame, 0),
+                                 GST_VIDEO_FRAME_COMP_DATA (&frame, 0));
 
   priv->frame[1] =
     video_texture_new_from_data (priv->ctx,
-                                 GST_VIDEO_INFO_COMP_WIDTH (&priv->info, 1),
-                                 GST_VIDEO_INFO_COMP_HEIGHT (&priv->info, 1),
+                                 GST_VIDEO_FRAME_COMP_WIDTH (&frame, 1),
+                                 GST_VIDEO_FRAME_COMP_HEIGHT (&frame, 1),
                                  COGL_PIXEL_FORMAT_RG_88,
-                                 priv->info.stride[1],
-                                 frame.data[1]);
+                                 GST_VIDEO_FRAME_COMP_STRIDE (&frame, 1),
+                                 GST_VIDEO_FRAME_COMP_DATA (&frame, 1));
 
   gst_video_frame_unmap (&frame);
 
@@ -1995,8 +2034,6 @@ clutter_gst_source_dispatch (GSource *source,
 
   if (buffer)
     {
-      clutter_gst_video_sink_upload_overlay (gst_source->sink, buffer);
-
       if (gst_buffer_get_video_gl_texture_upload_meta (buffer) != NULL) {
         GST_DEBUG_OBJECT (gst_source->sink,
                           "Trying to upload buffer %p with GL using renderer %s",
@@ -2012,6 +2049,8 @@ clutter_gst_source_dispatch (GSource *source,
           goto fail_upload;
       }
 
+      clutter_gst_video_sink_upload_overlay (gst_source->sink, buffer);
+
       priv->had_upload_once = TRUE;
 
       gst_buffer_unref (buffer);
@@ -2149,19 +2188,6 @@ clutter_gst_video_sink_dispose (GObject *object)
   self = CLUTTER_GST_VIDEO_SINK (object);
   priv = self->priv;
 
-  clear_frame_textures (self);
-
-  if (priv->renderer) {
-    priv->renderer->shutdown (self);
-    priv->renderer = NULL;
-  }
-
-  if (priv->clt_frame)
-    {
-      g_boxed_free (CLUTTER_GST_TYPE_FRAME, priv->clt_frame);
-      priv->clt_frame = NULL;
-    }
-
   if (priv->caps)
     {
       gst_caps_unref (priv->caps);
@@ -2212,10 +2238,10 @@ clutter_gst_video_sink_start (GstBaseSink *base_sink)
 static gboolean
 clutter_gst_video_sink_stop (GstBaseSink *base_sink)
 {
-  ClutterGstVideoSink *sink = CLUTTER_GST_VIDEO_SINK (base_sink);
-  ClutterGstVideoSinkPrivate *priv = sink->priv;
+  ClutterGstVideoSink *self = CLUTTER_GST_VIDEO_SINK (base_sink);
+  ClutterGstVideoSinkPrivate *priv = self->priv;
 
-  GST_INFO_OBJECT (sink, "Stop");
+  GST_INFO_OBJECT (self, "Stop");
 
   if (priv->source)
     {
@@ -2225,6 +2251,20 @@ clutter_gst_video_sink_stop (GstBaseSink *base_sink)
       priv->source = NULL;
     }
 
+  clear_frame_textures (self);
+
+  if (priv->renderer)
+    {
+      priv->renderer->shutdown (self);
+      priv->renderer = NULL;
+    }
+
+  if (priv->clt_frame)
+    {
+      g_boxed_free (CLUTTER_GST_TYPE_FRAME, priv->clt_frame);
+      priv->clt_frame = NULL;
+    }
+
   return TRUE;
 }
 
@@ -2299,7 +2339,7 @@ clutter_gst_video_sink_event (GstBaseSink * basesink, GstEvent * event)
     case GST_EVENT_FLUSH_START:
       g_mutex_lock (&gst_source->buffer_lock);
       if (gst_source->buffer) {
-        GST_DEBUG ("Freeing existing buffer %p", gst_source->buffer);
+        GST_DEBUG_OBJECT (sink, "Freeing existing buffer %p", gst_source->buffer);
         gst_buffer_unref (gst_source->buffer);
         gst_source->buffer = NULL;
       }
@@ -2625,6 +2665,7 @@ clutter_gst_video_sink_setup_pipeline (ClutterGstVideoSink *sink,
     {
       clutter_gst_video_sink_setup_conversions (sink, pipeline);
       clutter_gst_video_sink_setup_balance (sink, pipeline);
+      clutter_gst_video_sink_setup_transformation (sink, pipeline);
       priv->renderer->setup_pipeline (sink, pipeline);
     }
 }
